/*
 * integrity-score3.cpp
 *
 *  Created on: Jan 15, 2013
 *      Author: gpolles
 */

#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <cfloat>
#include <cmath>
#include <algorithm>

#include "PdbIoWrapper.h"

#define MAX_BEADS 1000000

using namespace std;

int find(int a, const vector<int>& v){
  for (size_t i = 0; i < v.size(); ++i) {
    if(a==v[i]) return i;
  }
  return -1;
}

inline double min(double a, double b){
  return (a<b? a:b);
}

inline double max(double a, double b){
  return (a>b? a:b);
}

inline int max(int a, int b){
  return (a>b? a:b);
}



class integrity_data{
public:
  integrity_data(): num_domains(0), num_jumps(0) {}
  void clear(){
    num_domains = 0;
    domains.resize(0);
    num_hits.resize(0);
    num_jumps = 0;
  }
  int num_domains;
  vector<int> domains;
  vector<int> num_hits;
  int num_jumps;

};

int main(int argc, char* argv[]) {



  // count number of beads
  int natoms = 0;
  vector<int> pstarts;
  vector<char> plabels;
  PdbIoWrapper pdb;
  pdb.openAsInput(argv[1]);
  PdbAtomInfo info,lastInfo;
  while(pdb.nextAtom(&info)) {
    if(info.chainId != lastInfo.chainId || info.resSeq < lastInfo.resSeq){
      pstarts.push_back(natoms);
      plabels.push_back(lastInfo.chainId);
    }
    natoms++;
    lastInfo = info;
  }
  pdb.close();
  int nproteins = int(pstarts.size());
  pstarts.push_back(natoms);

  // read domain file
  ifstream df(argv[2]);
  int idx,dm;
  vector<int> domain(natoms);
  int ndomains = 0;

  while(!(df >> idx >> dm).fail()){
    domain[idx] = dm;
    ndomains = max(ndomains,dm+1);
  }

  // compute integrity for each protein
  vector<double> integrities(nproteins);
  for (int ich = 0; ich < nproteins; ++ich) {
    vector<int> dcounts(ndomains,0);
    for (int i = pstarts[ich]; i < pstarts[ich+1]; ++i) {
      dcounts[domain[i]]++;
    }
    // find the max of dcounts
    double max_dcounts = 0;
    for (int i = 0; i < ndomains; ++i) {
      max_dcounts = max(double(dcounts[i]),max_dcounts);
    }
    double psize = pstarts[ich+1]- pstarts[ich];

    integrities[ich] = max_dcounts/psize;
  }

 // cout << natoms <<endl << nproteins<< endl << ndomains <<endl;

  // average
  double ave = 0;
  for(int i=0; i < nproteins; ++i){
    ave += integrities[i]/nproteins;
  }

//  vector<double> p_ave(4,0.0);
//  vector<int> p_cnt(4,0);
//  for (int i = 0; i < nproteins; ++i) {
//    p_ave[plabels[i]-'A'] += integrities[i];
//    p_cnt[plabels[i]-'A']++;
//  }




  // print and exit
  cout << ave ;
//  for (int i=0; i < 4; ++i){
//    cout << " " << p_ave[i]/p_cnt[i];
//  }
  cout << endl;
  return 0;
}



